/* eslint-disable no-undefined */
import { ETA } from "../classes/ETA.js";
import { getConsoleHeader } from "../util/functions/getters/getConsoleHeader.js";
import { isDebugEnabled } from "../util/functions/isDebugEnabled.js";
export class ProgressBar {
    constructor(options) {
        this.stream = process.stderr;
        this.waiterFrames = undefined;
        this.progressDotFrames = undefined;
        this.waiterId = undefined;
        this.progressId = undefined;
        this.linesToClear = 0;
        this.lineCount = 0;
        this.waiterFrameIndex = 0;
        this.progressFrameIndex = 0;
        this.total = 0;
        this.eta = new ETA(0, 0);
        this.value = 0;
        this.startTime = 0;
        this.lastProgressString = "";
        this.step = "";
        this.stepIndex = -1;
        this.steps = [];
        this.progressLine1 = "";
        const { progress: { totalSteps, steps, line1: progressLine1, spinnerColor: progressSpinnerColor }, waiter: { line1, line2, line3, doubleBackslash, interval, spinnerColor }, } = options;
        this.total = totalSteps;
        this.step = steps[0];
        this.stepIndex = 0;
        this.steps = steps;
        this.interval = interval ?? 250;
        this.progressSpinnerColor = progressSpinnerColor;
        this.progressLine1 = progressLine1;
        const waiterSpinner = this.getSpinners(spinnerColor);
        this.waiterFrames = [
            getConsoleHeader(line1 + waiterSpinner[0], line2, doubleBackslash ?? false, line3),
            getConsoleHeader(line1 + waiterSpinner[1], line2, doubleBackslash ?? false, line3),
            getConsoleHeader(line1 + waiterSpinner[2], line2, doubleBackslash ?? false, line3),
            getConsoleHeader(line1 + waiterSpinner[3], line2, doubleBackslash ?? false, line3),
        ];
        this.setProgressDots();
    }
    getSpinners(spinnerColor) {
        return [
            spinnerColor ? spinnerColor(".  ") : ".  ",
            spinnerColor ? spinnerColor(".. ") : ".. ",
            spinnerColor ? spinnerColor("...") : "...",
            spinnerColor ? spinnerColor("   ") : "   ",
        ];
    }
    startWaiter() {
        if (!this.waiterFrames || this.waiterId)
            return this;
        this.renderWaiter();
        this.waiterId = setInterval(this.renderWaiter.bind(this), this.interval);
        return this;
    }
    frameWaiter() {
        if (!this.waiterFrames)
            return "";
        const frames = this.waiterFrames, frame = frames[this.waiterFrameIndex];
        this.waiterFrameIndex = ++this.waiterFrameIndex % frames.length;
        this.lineCount = frame.split("\n").length;
        return frame;
    }
    renderWaiter() {
        if (!this.stream.isTTY)
            return this;
        if (isDebugEnabled())
            return this;
        this.clear();
        this.stream.write(this.frameWaiter());
        this.linesToClear = this.lineCount;
        return this;
    }
    startProgress() {
        if (this.progressId)
            return this;
        if (this.waiterId)
            this.stopWaiter();
        this.startTime = Date.now();
        this.lastProgressString = "";
        this.eta = new ETA(this.startTime, this.value);
        this.renderProgress();
        this.progressId = setInterval(this.renderProgress.bind(this), this.interval);
        return this;
    }
    incrementProgress(value = 1) {
        this.value += value;
        this.eta.update(Date.now(), this.value, this.total);
    }
    renderProgress() {
        const frameString = this.frameProgress();
        if (!this.stream.isTTY)
            return this;
        if (isDebugEnabled())
            return this;
        if (this.lastProgressString !== frameString) {
            this.updateETA();
            if (this.lastProgressString !== "")
                this.clear();
            this.stream.write(frameString);
            this.linesToClear = this.lineCount;
            this.lastProgressString = frameString;
        }
        return this;
    }
    frameProgress() {
        const frame = getConsoleHeader(this.dotsProgressWaiter(), `${this.getProgressString()} ${this.getProgressPercentageString()} | ETA: ${this.getETAString()}`, false, `Step: ${this.value}/${this.total} (${this.step})`);
        this.lineCount = frame.split("\n").length;
        return frame;
    }
    dotsProgressWaiter() {
        if (!this.progressDotFrames)
            return "";
        const frames = this.progressDotFrames, dots = frames[this.progressFrameIndex];
        this.progressFrameIndex = ++this.progressFrameIndex % frames.length;
        return dots;
    }
    getETAString() {
        const t = this.eta.getTime();
        function round(input) {
            return 5 * Math.round(input / 5);
        }
        if (typeof t === "string")
            return t;
        // > 1h ?
        if (t > 3600)
            return `${this.autopadding(Math.floor(t / 3600), 2)}h${this.autopadding(round((t % 3600) / 60), 2)}m`;
        // > 60s ?
        else if (t > 60)
            return `${this.autopadding(Math.floor(t / 60), 2)}m${this.autopadding(round(t % 60), 2)}s`;
        // > 10s ?
        else if (t > 10)
            return `${this.autopadding(round(t), 2)}s`;
        // default: don't apply round to multiple
        else
            return `${this.autopadding(t, 2)}s`;
    }
    autopadding(v, length) {
        return `${v}`.slice(-length);
    }
    getProgressPercentageString() {
        const percentage = Math.floor(this.getProgress() * 100);
        return `${this.autopadding(percentage, 3)}%`;
    }
    getProgressString() {
        const barWidth = 41, completeString = Array.from({ length: barWidth }).join("\u2588"), incompleteString = Array.from({ length: barWidth }).join("\u2591"), completeSize = Math.round(this.getProgress() * barWidth), incompleteSize = barWidth - completeSize;
        return completeString.slice(0, completeSize) + incompleteString.slice(0, incompleteSize);
    }
    getProgress() {
        let progress = this.value / this.total;
        if (Number.isNaN(progress))
            progress = 1;
        progress = Math.min(Math.max(progress, 0), 1);
        return progress;
    }
    clear() {
        if (!this.stream.isTTY)
            return this;
        this.stream.cursorTo(0);
        for (let index = 0; index < this.linesToClear; index++) {
            if (index > 0)
                this.stream.moveCursor(0, -1);
            this.stream.clearLine(1);
        }
        return this;
    }
    stop() {
        this.stopWaiter();
        this.stopProgress();
        return this;
    }
    stopWaiter() {
        if (!this.waiterId)
            return this;
        clearInterval(this.waiterId);
        this.waiterId = undefined;
        this.waiterFrameIndex = 0;
        this.clear();
        return this;
    }
    stopProgress() {
        if (!this.progressId)
            return this;
        clearInterval(this.progressId);
        this.progressId = undefined;
        this.clear();
        return this;
    }
    updateETA() {
        this.eta.update(Date.now(), this.value, this.total);
    }
    setStep(step) {
        if (step <= this.stepIndex || !this.steps[step])
            return this;
        this.stepIndex = step;
        this.step = this.steps[step];
        this.updateETA();
        return this;
    }
    setProgressLine1(line) {
        this.progressLine1 = line;
        this.setProgressDots();
        return this;
    }
    setProgressDots() {
        const progressDot = this.getSpinners(this.progressSpinnerColor);
        this.progressDotFrames = [
            this.progressLine1 + progressDot[0],
            this.progressLine1 + progressDot[1],
            this.progressLine1 + progressDot[2],
            this.progressLine1 + progressDot[3],
        ];
    }
}
//# sourceMappingURL=ProgressBar.js.map